load ../lib/common

# bats test_tags=network,sudo
@test "librechroot creates repo for new chroots" {
	testsudo librechroot -l "$BATS_TEST_NAME" run test -r /repo/repo.db
}

# bats test_tags=network,sudo
@test "librechroot cleans the local repo correctly" {
	testsudo librechroot -l "$BATS_TEST_NAME" make
	testsudo librechroot -l "$BATS_TEST_NAME" clean-repo
	testsudo librechroot -l "$BATS_TEST_NAME" run test -r /repo/repo.db
	# TODO: inspect /repo/* more
}

# bats test_tags=network,sudo
@test "librechroot respects exit status if out isnt a tty" {
	set -o pipefail
	{ testsudo librechroot -l "$BATS_TEST_NAME" run bash -c 'exit 3' | cat; } || status=$?

	[[ $status == 3 ]]
}

# bats test_tags=network,sudo
@test "librechroot creates ca certificates" {
	testsudo librechroot -l "$BATS_TEST_NAME" run test -r /etc/ssl/certs/ca-certificates.crt
}

# bats test_tags=network,sudo
@test "librechroot disables networking when requested" {
	    testsudo librechroot -l "$BATS_TEST_NAME"    run curl https://repo.parabola.nu/ >/dev/null
	not testsudo librechroot -l "$BATS_TEST_NAME" -N run curl https://repo.parabola.nu/ >/dev/null
}

# bats test_tags=network,sudo
@test "librechroot handles CHROOTEXTRAPKG correctly" {
	not testsudo librechroot -l "$BATS_TEST_NAME" run lsof
	echo "CHROOTEXTRAPKG=(lsof)" >> "$XDG_CONFIG_HOME"/libretools/chroot.conf
	testsudo librechroot -l "$BATS_TEST_NAME" install-name lsof
	testsudo librechroot -l "$BATS_TEST_NAME" clean-pkgs
	testsudo librechroot -l "$BATS_TEST_NAME" run lsof
	echo "CHROOTEXTRAPKG=()" >> "$XDG_CONFIG_HOME"/libretools/chroot.conf
	testsudo librechroot -l "$BATS_TEST_NAME" clean-pkgs
	not testsudo librechroot -l "$BATS_TEST_NAME" run lsof
}

# bats test_tags=network,sudo
@test "librechroot obeys depends in PKGBUILD" {
	# NB: We chown `/startdir` because chcleanup runs `makepkg
	# --printsrcinfo > .SRCINFO` as the user that owns
	# `/startdir/PKGBUILD`, and `makepkg` doesn't like to be run
	# as root.  Unfortunately, this test runs with a chroot that
	# doesn't have the `builduser` set up, and it'd be a pain to
	# set it up.  So choose an arbitrary user from
	# `/usr/lib/sysusers.d/arch.conf`.
	local builduser_uid="${SUDO_UID:-$UID}"
	local builduser_gid="$(id -g "$builduser_uid")"
	testsudo librechroot -l "$BATS_TEST_NAME" run sh -c "
		echo 'builduser:x:${builduser_gid}:'                                            >>/etc/group
		echo 'builduser:x:${builduser_uid}:${builduser_gid}:builduser:/build:/bin/bash' >>/etc/passwd
		echo 'builduser:!!:$(( $(date -u +%s) / 86400 ))::::::'                         >>/etc/shadow"

	# Pre-flight: Make sure `lsof` isn't in the base install
	not testsudo librechroot -l "$BATS_TEST_NAME" run lsof
	# Make sure librechroot removes `lsof` when the PKGBUILD is not present.
	testsudo librechroot -l "$BATS_TEST_NAME" install-name lsof
	testsudo librechroot -l "$BATS_TEST_NAME" run lsof
	testsudo librechroot -l "$BATS_TEST_NAME" clean-pkgs
	not testsudo librechroot -l "$BATS_TEST_NAME" run lsof
	# Make sure librechroot keeps `lsof` when the PKGBUILD is present.
	testsudo librechroot -l "$BATS_TEST_NAME" install-name lsof
	testsudo librechroot -l "$BATS_TEST_NAME" run sh -c 'printf "%s\n" "$1" > /startdir/PKGBUILD' -- "$(cat fixtures/librechroot/PKGBUILD-arches)"
	testsudo librechroot -l "$BATS_TEST_NAME" run chown -R builduser: /startdir
	testsudo librechroot -l "$BATS_TEST_NAME" clean-pkgs
	testsudo librechroot -l "$BATS_TEST_NAME" run lsof
}

# bats test_tags=network,sudo
@test "librechroot can install libretools with chcleanup" {
	# NB: We chown `/startdir` because chcleanup runs `makepkg
	# --printsrcinfo > .SRCINFO` as the user that owns
	# `/startdir/PKGBUILD`, and `makepkg` doesn't like to be run
	# as root.
	local builduser_uid="${SUDO_UID:-$UID}"
	local builduser_gid="$(id -g "$builduser_uid")"
	testsudo librechroot -l "$BATS_TEST_NAME" run sh -c "
		echo 'builduser:x:${builduser_gid}:'                                            >>/etc/group
		echo 'builduser:x:${builduser_uid}:${builduser_gid}:builduser:/build:/bin/bash' >>/etc/passwd
		echo 'builduser:!!:$(( $(date -u +%s) / 86400 ))::::::'                         >>/etc/shadow"

	not testsudo librechroot -l "$BATS_TEST_NAME" run test -f /usr/bin/chcleanup
	testsudo librechroot -l "$BATS_TEST_NAME" run mkdir /startdir
	testsudo librechroot -l "$BATS_TEST_NAME" run sh -c 'printf "%s\n" "$1" > /startdir/PKGBUILD' -- "$(cat fixtures/librechroot/PKGBUILD-libretools)"
	testsudo librechroot -l "$BATS_TEST_NAME" run chown -R builduser: /startdir
	testsudo librechroot -l "$BATS_TEST_NAME" clean-pkgs
	testsudo librechroot -l "$BATS_TEST_NAME" run test -f /usr/bin/chcleanup
}

@test "librechroot displays help as normal user" {
	rm -rf "$XDG_CONFIG_HOME"
	LC_ALL=C librechroot help >$tmpdir/stdout 2>$tmpdir/stderr

	[[ "$(sed 1q $tmpdir/stdout)" =~ Usage:.* ]]
	empty $tmpdir/stderr
}

@test "librechroot otherwise fails as normal user" {
	librechroot -l "$BATS_TEST_NAME" run true >$tmpdir/stdout 2>$tmpdir/stderr || status=$?

	[[ $status != 0 ]]
	empty $tmpdir/stdout
	not empty $tmpdir/stderr
}

@test "librechroot displays help and fails with 0 args" {
	LC_ALL=C librechroot -l "$BATS_TEST_NAME" >$tmpdir/stdout 2>$tmpdir/stderr || status=$?

	[[ $status != 0 ]]
	empty $tmpdir/stdout
	[[ "$(sed -n 2p $tmpdir/stderr)" =~ Usage:.* ]]
}

# bats test_tags=network,sudo
@test "librechroot obeys the n flag" {
	not test -f "$chrootdir/$BATS_TEST_NAME/$BATS_TEST_NAME/$BATS_TEST_NAME.stamp"

	testsudo librechroot -n "$BATS_TEST_NAME" -l "$BATS_TEST_NAME" run touch /"$BATS_TEST_NAME.stamp"

	test -f "$chrootdir/$BATS_TEST_NAME/$BATS_TEST_NAME/$BATS_TEST_NAME.stamp"
}

# requires sudo so we know it's not failing because it needs root
# bats test_tags=sudo
@test "librechroot fails for unknown commands" {
	testsudo librechroot phony >$tmpdir/stdout 2>$tmpdir/stderr || status=$?

	[[ $status != 0 ]]
	empty $tmpdir/stdout
	not empty $tmpdir/stderr
}

# requires sudo so we know it's not failing because it needs root
# bats test_tags=sudo
@test "librechroot fails for unknown flags" {
	testsudo librechroot -q >$tmpdir/stdout 2>$tmpdir/stderr || status=$?

	[[ $status != 0 ]]
	empty $tmpdir/stdout
	not empty $tmpdir/stderr
}

# bats test_tags=sudo
@test "librechroot fails when syncing a copy with itself" {
	testsudo timeout 5 librechroot -l root sync  || status=$?
	case $status in
		0|124|137) # success|timeout+TERM|timeout+KILL
			false;;
		*)
			true;;
	esac
}

# bats test_tags=network,sudo
@test "librechroot deletes copies" {
	testsudo librechroot -l "$BATS_TEST_NAME" make
	test -d "$chrootdir/default/$BATS_TEST_NAME"
	testsudo librechroot -l "$BATS_TEST_NAME" delete
	not test -e "$chrootdir/default/$BATS_TEST_NAME"
}

# bats test_tags=network,sudo,btrfs
@test "librechroot deletes subvolumes recursively" {
	[[ "$(stat -f -c %T "$chrootdir" 2>/dev/null || true)" == 'btrfs' ]]
	testsudo librechroot -l "$BATS_TEST_NAME" make
	testsudo librechroot -l "$BATS_TEST_NAME" install-name btrfs-progs
	test -d "$chrootdir/default/$BATS_TEST_NAME"
	not test -e "$chrootdir/default/$BATS_TEST_NAME/var/subvolume"
	testsudo librechroot -l "$BATS_TEST_NAME" run btrfs subvolume create /var/subvolume
	test -d "$chrootdir/default/$BATS_TEST_NAME/var/subvolume"
	testsudo librechroot -l "$BATS_TEST_NAME" delete
	not test -e "$chrootdir/default/$BATS_TEST_NAME"
}

# bats test_tags=network,sudo
@test "librechroot cleans up TMPDIR" {
	local dir="$tmpdir/tmp"
	mkdir -- "$dir"

	TMPDIR=$dir testsudo librechroot -l "$BATS_TEST_NAME" -A x86_64 make

	# Make sure $dir is now empty
	rmdir -- "$dir"
}
